@@ -7,8 +7,8 @@ describe('process', () => {
       expect(typeof _process).toBe('object');
     });
     it('.getuid() and .getgid()', () => {
-      expect(typeof proc.getuid()).toBe('number');
-      expect(typeof proc.getgid()).toBe('number');
+      expect(typeof proc.getuid?.() ?? 0).toBe('number');
+      expect(typeof proc.getgid?.() ?? 0).toBe('number');
     });
     it('.cwd()', () => {
       expect(typeof proc.cwd()).toBe('string');
